/*
 * Copyright (C) 2022 Associated Universities, Inc. Washington DC, USA.
 *
 * This file is part of NRAO Workspaces.
 *
 * Workspaces is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Workspaces is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
 */

package db

import (
	"database/sql"
	"fmt"
	_ "github.com/lib/pq"
)

type UserInfo struct {
	Name  string
	Email string
	IsAod bool
}

/**
 * Check if an error happened and panic on it if so
 *
 * @param err An error object to report
 **/
func checkError(err error) {
	if err != nil {
		panic(err)
	}
}

//AddUser
/**
 * Add a user to the qa_staff table in the database
 *
 * @param name a string with the name of the person to be added
 * @param is_aod a bool to check if the user is an AOD or not
 * @param db_info a DbInfo type with information to connect to the database
 **/
func AddUser(user UserInfo, connectionInfo DbInfo) {
	// Get a connection to the database
	db := GetConnection(connectionInfo)

	defer func(db *sql.DB) {
		err := db.Close()
		checkError(err)
	}(db)

	group := "DA"
	if user.IsAod {
		group = "AOD"
	}

	// Add the user
	insertStatement := `insert into "qa_staff"("user_name", "group", "available", "email") values($1, $2, $3, $4)`
	_, err := db.Exec(insertStatement, user.Name, group, true, user.Email)
	checkError(err)

	fmt.Println("Added " + user.Name + " to qa_staff")
}

//RemoveUser
/**
 * Remove a user from the qa_staff table in the database
 *
 * @param name a string with the name of the person to be removed
 * @param db_info a DbInfo type with information to connect to the database
 **/
func RemoveUser(user UserInfo, connectionInfo DbInfo) {
	// Get a connection to the database
	db := GetConnection(connectionInfo)

	defer func(db *sql.DB) {
		err := db.Close()
		checkError(err)
	}(db)

	// Remove the user
	deleteStatement := `delete from "qa_staff" where "user_name"=$1`
	_, err := db.Exec(deleteStatement, user.Name)
	checkError(err)

	fmt.Println("Removed " + user.Name + " from qa_staff")
}
