"""
Facility for removing old or QAFailed files and directories from the Workspaces lustre areas
* Download area clean up is handled by the AAT/PPI mr_clean system
"""
import argparse
import logging
import os
import pathlib
import shutil
import sys
from argparse import ArgumentParser
from enum import Enum
from typing import Dict, List

import requests
from pycapo import CapoConfig

logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)
logger.addHandler(logging.StreamHandler(sys.stdout))


class Areas(Enum):
    CACHE = "cache"
    SPOOL = "spool"
    STAGING = "staging"
    STORAGE = "storage"


def _get_settings(profile: str) -> Dict[str, str]:
    """
    Get annihilator capo settings

    :return:
    """
    config = CapoConfig(profile=profile)

    annihilator_settings = config.settings("edu.nrao.workspaces.AnnihilatorSettings")
    ingestion_settings = config.settings("edu.nrao.workspaces.IngestionSettings")

    workflow_url = config.settings("edu.nrao.workspaces.WorkflowSettings").serviceUrl
    cache_directory = config.settings("edu.nrao.workspaces.DeliverySettings").cacheWeblogDirectory
    spool_directory = config.settings("edu.nrao.workspaces.ProcessingSettings").rootDirectory
    staging_directory = ingestion_settings.stagingDirectory
    storage_directory = ingestion_settings.storageDirectory

    return {
        "workflow_url": workflow_url,
        "cache": cache_directory,
        "spool": spool_directory,
        "storage": storage_directory,
        "staging": staging_directory,
        "cache_days": annihilator_settings.keepCacheForDays,
        "spool_days": annihilator_settings.keepSpoolForDays,
        "staging_days": annihilator_settings.keepStagingForDays,
        "storage_days": annihilator_settings.keepStorageForDays,
    }


def _arg_parser() -> ArgumentParser:
    """
    Create parser for annihilator

    :return:
    """
    arg_parser = ArgumentParser(
        description="Workspaces Directory Annihilator",
        formatter_class=argparse.RawTextHelpFormatter,
    )
    arg_parser.add_argument(
        "-p",
        "--profile",
        action="store",
        required=True,
        help="Specify which environment to run on. Required. Example: dsoc-dev",
    )
    arg_parser.add_argument(
        "-a",
        "--all",
        action="store_true",
        required=False,
        help="Run annihilation on workspaces spool, staging, and storage areas.",
    )
    arg_parser.add_argument(
        "-c",
        "--cache",
        action="store_true",
        required=False,
        help="Run annihilation on workspaces cache area.",
    )
    return arg_parser


class Annihilator:
    """
    Class for removing workspaces directories that are no longer in use and past a given date

    - Retrieves a list of stale request directories to annihilate
    - Checks if directories still exist
        (DAs might manually clean up directories until scheduling system is available)
    - If directory exists in the workspaces' area for the specified profile, annihilates directory
        and sets 'cleaned' flag on the associated request
    - If directory doesn't exist, sets 'cleaned' flag on associated request
    """

    def __init__(self):
        self._args = _arg_parser().parse_args()
        self._settings = _get_settings(self._args.profile)

    def determine_path(self, area_to_clean: str) -> str:
        """
        return path to the location from capo settings

        :param area_to_clean: workspaces lustre area to clean, i.e. "spool"
        :return: path string
        """
        return self._settings.get(area_to_clean.lower())

    @staticmethod
    def path_valid(constructed_path: str, base_path: str) -> bool:
        """
        Check if the constructed path is valid for annihilation

        :param constructed_path: path to verify
        :param base_path: base path which directory should be annihilated from
        :return: True if path is valid for annihilation
        """

        return (
            constructed_path != base_path
            and constructed_path != base_path + "/"
            and len(constructed_path) > len(base_path)
        )

    def get_days_to_keep(self, area_to_clean: str) -> str:
        """
        Return the number of days to keep directories in a specified area from capo settings

        :param area_to_clean: workspaces lustre area to clean, i.e. "spool"
        :return:
        """
        key = area_to_clean.lower() + "_days"
        return self._settings.get(key)

    def get_stale_directories(self, area_to_clean: str) -> List[str]:
        """
        It is possible for processing to take multiple days to several weeks. While a job is running in HTCondor, it's
        parent workspaces directory in the spool area won't update other than the HTCondor log file(s).
        We want to be sure that only stale directories are annihilated.

        It is also possible for the dev and test systems to pick up production directories due to database copying.
        For this reason we only want to use the name of the directory itself and strip off the rest of the path.

        :param area_to_clean: workspaces lustre area to clean, i.e. "spool"
        :return: A list of directory names to be removed
        """

        annihilation_url = (
            self._settings.get("workflow_url") + "/workflows/requests/stale/" + self.get_days_to_keep(area_to_clean)
        )
        response = requests.get(annihilation_url)
        dir_list = response.json()["stale_dirs"]

        name_list = []
        # catch empty string response
        if dir_list:
            for stale in dir_list.split(","):
                # catch empty string directory or whitespace only directory or VLASS directory (VLASS will handle VLASS)
                if stale and not stale.isspace() and not stale.startswith("VLASS"):
                    name_list.append(pathlib.Path(str(stale)).name)

        return name_list

    def annihilate_directory(self, area: str, directory: str):
        """
        Annihilate specified directory

        :param area: area being cleaned
        :param directory: directory to remove
        :return:
        """

        shutil.rmtree(directory)

        # After annihilation set cleaned flag...
        if area == Areas.SPOOL.value:
            logger.info(f"Annihilation complete for {directory}, setting 'cleaned' flag on workflow request...")
            path = self.determine_path(area)
            clean_me = path + "/" + directory
            self.set_cleaned(clean_me)
        else:
            logger.info(f"Annihilation complete for {directory} in {area}.")

    def set_cleaned(self, directory: str):
        """
        Set the 'cleaned' flag on the request of the specified directory

        :param directory: the directory that was annihilated
        :return:
        """
        clean_url = self._settings.get("workflow_url") + "/workflows/requests/cleaned"
        json_payload = {"directory": directory, "update_to": "True"}

        return requests.put(clean_url, json=json_payload)

    def annihilate(self, area_to_clean: str):
        """
        Remove stale directories from targeted workspaces areas

        :return:
        """
        stale = self.get_stale_directories(area_to_clean)

        # If stale directories were found, annihilate them
        if stale:
            path = self.determine_path(area_to_clean)

            # change to area to clean
            os.chdir(path)

            for directory in stale:
                dir_path = path + "/" + directory
                if self.path_valid(dir_path, path):
                    if not pathlib.Path(dir_path).exists():
                        if area_to_clean == Areas.SPOOL.value:
                            logger.info(
                                f"directory {dir_path} not found, setting 'cleaned' flag on workflow request..."
                            )
                            self.set_cleaned(dir_path)
                        else:
                            logger.info(f"directory {dir_path} not found in {area_to_clean}, skipping...")
                    else:
                        logger.info(f"found directory {directory}, annihilating...")
                        self.annihilate_directory(area_to_clean, directory)
                else:
                    logger.error(f"Error: Received invalid directory '{dir_path}' as input for annihilation. Aborting.")
                    exit(1)

    def run(self):
        if self._args.all:
            logger.info(f"Starting cleaning of Workspaces Spool for profile {self._args.profile}")
            self.annihilate(Areas.SPOOL.value)

            logger.info(f"Starting cleaning of Workspaces Staging for profile {self._args.profile}")
            self.annihilate(Areas.STAGING.value)

            logger.info(f"Starting cleaning of Workspaces Storage for profile {self._args.profile}")
            self.annihilate(Areas.STORAGE.value)
            logger.info(
                f"Finished cleaning Workspaces Spool, Staging, and Storage areas for profile {self._args.profile}!"
            )
        if self._args.cache:
            logger.info(f"Starting cleaning of Workspaces Cache for profile {self._args.profile}")
            self.annihilate(Areas.CACHE.value)
            logger.info(f"Finished cleaning Workspaces Cache for profile {self._args.profile}!")
        else:
            logger.info(f"Starting cleaning of Workspaces Spool for profile {self._args.profile}")
            self.annihilate(Areas.SPOOL.value)
            logger.info(f"Finished cleaning Workspaces Spool for profile {self._args.profile}!")


def main():
    Annihilator().run()
